#! /usr/bin/perl -w
use strict;
use utf8;

# MakeAMSFormats.pl
# make amsppt.fmt, pamstex.fmt, pamsppt.fmt
# amstex, amsppt, pamstex, pamsppt 様式を作る.
# 直接実行, 管理者用パスワードが必要.

# ver 6.1c13;	MacTeX 2011 support (ascii-jplain.tex)
# ver 6.0c23;	self boot AppleScript
# ver 6.0c16;	use $ENV{"JTeXEditTeXBinPath"}
# ver 6.0c16;	support texlive2008
# ver 5.2a01;	support bigptex060120 (ogawa)
# ver 5.1b04;	use external sudo, support ptex package 2005v2.dmg (kiriki)
# ver 5.1a16;	untaint some $ENV's for OS X 10.4.
#				require password for modifications.

my $arg = shift;
unless ($arg and $arg eq 'Privileged') {
	my $path = $0;
	my $bin_dir = '/usr/local/bin';
	$bin_dir = $ENV{"JTeXEditTeXBinPath"} if ($ENV{"JTeXEditTeXBinPath"});
	my $script =<<EOS;
set myResult to do shell script "perl '$path' Privileged '$bin_dir'" with administrator privileges
tell application "JTeXEdit" to Scan ShellTeX
return myResult
EOS
	my $answer = `osascript <<'_HERE_'\n$script\n_HERE_`;
	print "$answer\n";
	exit 0;
}

# Real job in administrator privileges

my $bin_dir = shift;

my $ForceFlag = 1; # do overwrite if 1
my $TestFlag = ''; # don't write if 1

my $kpsewhich = "$bin_dir/kpsewhich";
die "kpsewhich not found" unless (-e $kpsewhich);
print "\nusing $kpsewhich\n";

sub FindPath($) {
	my ($name) = @_;
	my $path = `$kpsewhich $name`;
	chomp $path;
	return $path;
}

sub ReadFile($) {
	my ($file) = @_;
	return '' unless (-e $file);
	open(IN, "<:bytes", $file) or die "can't read '$file'";
	my $text = join('', <IN>);
	close(IN);
	return $text;
}

sub WriteFile($$) {
	my ($file, $text) = @_;
	if (open(IN, "<:bytes", $file)) {
		my $oldtext = join('', <IN>);
		close(IN);
		return if ($oldtext eq $text);
	}
	if ($TestFlag) {
		print "\nSkip Writing to $file\n****\n$text\n****\n";
		return;
	}
	open(OUT, ">:bytes", $file) or die "can't write '$file'";
	print OUT $text or die $!;
	close(OUT);
	print "file '$file' written\n";
}

# search amstex.ini
my $amstex_ini_dir = '';
{
	my $path = FindPath("amstex.ini");
	die "amstex.ini not found." unless (-e $path);
	($amstex_ini_dir) = $path =~ m|^(.*)/amstex.ini$|;
}

# search jplain.tex
my $jplain_tex = '';
{
	my $path = FindPath("jplain.tex");
	$jplain_tex = "jplain.tex" if ($path);
	unless ($jplain_tex) {
		$path = FindPath("ascii-jplain.tex");
		$jplain_tex = "ascii-jplain.tex" if ($path);
	}
	die "jplain.tex not found." unless ($jplain_tex);
}

# search web2c in texmf-local or texmf
sub FindWeb2C() {
	my $web2c = '';
	my $text = `$kpsewhich -show-path 'web2c files'`;
	my @list = split(/:|!/, $text);
	for (@list) {
		if (m|/texmf-local/web2c$|) {
			$web2c = $_;
			last;
		} elsif (m|^/usr/.*/texmf/web2c$|) {
			$web2c = $_;
		}
	}
	die "web2c not found." unless $web2c;
	`mkdir -p '$web2c'` unless (-e $web2c);
	die "web2c not found." unless (-e $web2c);
	print "web2c is $web2c\n";
	return $web2c;
}

# make ams ini files from amstex.ini

my %texmfcnf = ();
my @makeformat = ();

my $amstex_ini = ReadFile("$amstex_ini_dir/amstex.ini")
or die "can't read $amstex_ini_dir/amstex.ini.";
if ($ForceFlag or not (-e "$bin_dir/amstex")) {
	print "will make amstex.\n";
	push(@makeformat, 'amstex');
} else {
	print "$bin_dir/amstex already exists.\n";
}

my $amsppt_ini = $amstex_ini;
$amsppt_ini =~ s/^%+\\documentstyle\{amsppt\}/\\documentstyle\{amsppt\}/m
or $amsppt_ini =~ s/^\\dump\b/\\documentstyle\{amsppt\}\n\\dump/m;
WriteFile("$amstex_ini_dir/amsppt.ini", $amsppt_ini);
if ($ForceFlag or not (-e "$bin_dir/amsppt")) {
	$texmfcnf{'TEXINPUTS.amsppt'} = '.;$TEXMF/tex/{amstex,plain,generic,}//';
	print "will make amsppt.\n";
	push(@makeformat, 'amsppt');
} else {
	print "$bin_dir/amsppt already exists.\n";
}

$amstex_ini =~ s/^\\input plain/\\input $jplain_tex/m;
WriteFile("$amstex_ini_dir/pamstex.ini", $amstex_ini);
if ($ForceFlag or not (-e "$bin_dir/pamstex")) {
	$texmfcnf{'TEXINPUTS.pamstex'} = '.;$TEXMF/{ptex,tex}/{amstex,plain,generic,}//';
	print "will make pamstex.\n";
	push(@makeformat, 'pamstex');
} else {
	print "$bin_dir/pamstex already exists.\n";
}

$amsppt_ini =~ s/^\\input plain/\\input $jplain_tex/m;
WriteFile("$amstex_ini_dir/pamsppt.ini", $amsppt_ini);
if ($ForceFlag or not (-e "$bin_dir/pamsppt")) {
	$texmfcnf{'TEXINPUTS.pamsppt'} = '.;$TEXMF/{ptex,tex}/{amstex,plain,generic,}//';
	print "will make pamsppt.\n";
	push(@makeformat, 'pamsppt');
} else {
	print "$bin_dir/pamsppt already exists.\n";
}

# rewrite texmf.cnf
if (%texmfcnf) {
	# $texmfcnf{'TEXINPUTS.iniptex'} = '$ptex_inputs';
	my $texmf_cnf = FindPath("texmf.cnf");
	die "texmf.cnf not found" unless (-e $texmf_cnf);
	my $text = ReadFile($texmf_cnf)
	or die "$texmf_cnf not found.";
	my $rewrite = '';
	my $insert = '';
	for my $item (keys %texmfcnf) {
		if ($text =~ /^$item\s*=\s*(.*)$/m) {
			next if (! $ForceFlag);
			next if ($1 eq $texmfcnf{$item});
			$text =~ s/^($item\s*=.*)$/%$1\n$item = $texmfcnf{$item}\n/m;
			$rewrite = 1;
		} else {
			$insert .= "$item = $texmfcnf{$item}\n";
		}
	}
	if ($insert) {
		unless ($text =~ s/(\n%.*MakeFormat.*?\n\n)/$1$insert/s) {
			$text =~ s/(\nTEXINPUTS\.ptex\s*=.*?\n\n)/$1% by MakeFormat\n$insert\n/s
			or $text .= "\n% by MakeFormat\n$insert\n"; # texlive-2008
		}
	}
	if ($insert or $rewrite) {
		my $name = $texmf_cnf;
		my $idx = 0;
		while (-e "$name.$idx") { ++$idx; }
		unless ($TestFlag) {
			rename($texmf_cnf, "$name.$idx")
			or die "can't rename '$texmf_cnf'";
		}
		print "rewrite $texmf_cnf,\n backup: $name.$idx\n";
		WriteFile($texmf_cnf, $text);
	}
}

# make format
my $web2c = '';
for my $item (@makeformat) {
	print "making $item\n";
	my $initex = ($item =~ /^p/) ? 'ptex' : 'tex';
	my $fmt_path = FindPath("$item.fmt");
	my $log_path = '';
	if ($fmt_path) {
		$log_path = $fmt_path;
		$log_path =~ s/\.fmt$/.log/;
	} else {
		$web2c = FindWeb2C unless $web2c;
		$fmt_path = "$web2c/$item.fmt";
		$log_path = "$web2c/$item.log";
	}
	chdir($amstex_ini_dir) or die "can't chdir '$amstex_ini_dir'";
	die "$item.ini was not generated." unless (-e "$item.ini");
	print "process '$bin_dir/$initex -ini -interaction batchmode $item.ini'\n";
	`$bin_dir/$initex -ini -interaction batchmode $item.ini`; # wait
	print "process-end\n";
	rename("$item.fmt", $fmt_path) or die "can't rename '$item.fmt' to '$fmt_path'";
	rename("$item.log", $log_path) or die "can't rename '$item.log' to '$log_path'";
	unless (-e "$bin_dir/$item") {
		print "symlink $bin_dir/$item -> $bin_dir/$initex\n";
		symlink("$bin_dir/$initex", "$bin_dir/$item") or warn "can't symlink '$bin_dir/$item'\n";
	}
}

if (@makeformat) {
	(system("$bin_dir/mktexlsr") == 0) or die "can't $bin_dir/mktexlsr";
}

print "completed\n";
