#!/usr/bin/perl -w
use strict;
use utf8;
use open IO => ":utf8";
use Encode;

# 使用法:
#	2 つのファイルまたはフォルダをデータとして比較する．
#	一度に渡せないときは "指令/順序処理..." すること．

my ($Obj1, $Obj2) = @ARGV;
utf8::decode($Obj1);
utf8::decode($Obj2);
die "Error: Pass two objects.\n" unless defined($Obj2);
my $DirMode = -d $Obj1 or -d $Obj2;
print "/usr/bin/diff -q '$Obj1' '$Obj2'\n%%%%%%%%%%%%%%%%\n";

sub NewFile($) {
	my ($path) = @_;
	my $tmp = 0;
	while (-e "${path}_$tmp") { ++$tmp; }
	return "${path}_$tmp";
}

my %Table;
my @SameName;
sub AddFiles($);
sub AddFiles($) {
	my ($path) = @_;
	if (-d $path) {
		opendir(DIR, $path) or die $!;
		my @items = readdir(DIR);
		for (@items) { utf8::decode($_); }
		closedir(DIR);
		for my $item (@items) {
			next if ($item =~ /^\./);
			next if (-l "$path/$item");
			AddFiles("$path/$item");
		}
	} else {
		$path =~ m|/([^/]*)$|;
		if (defined($Table{$1})) {
			push(@SameName, $path);
		} else {
			$Table{$1} = $path;
		}
	}
}

my %Sames = ();
sub MakeTable($) {
	my ($path) = @_;
	%Table = ();
	@SameName = ();
	AddFiles($path);
	for my $item (@SameName) {
		$item =~ m|/([^/]*)$|;
		if (defined($Table{$1})) {
			push(@{$Sames{$1}}, $Table{$1});
			delete $Table{$1};
		}
		push(@{$Sames{$1}}, $item);
	}
}
MakeTable($Obj1) if $DirMode;

my (@Diffs, @Sames);

sub Compare($$) {
	my ($file1, $file2) = @_;
	my $diff = `/usr/bin/diff -q '$file1' '$file2'`;
	if ($diff =~ /^Files.*differ$/) {
		push(@Diffs, "differ: '$file1', '$file2'\n");
	} elsif (length($diff) == 0) {
		push(@Sames, "same: '$file1', '$file2'\n");
	} else {
		warn "format error: $diff";
	}
}

my @SameResult = ();
my @NotFound = ();
sub ParseFiles($);
sub ParseFiles($) {
	my ($path) = @_;
	if (-d $path) {
		opendir(DIR, $path) or die $!;
		my @items = readdir(DIR);
		closedir(DIR);
		for (@items) { utf8::decode($_); }
		for my $item (@items) {
			next if ($item =~ /^\./);
			next if (-l "$path/$item");
			ParseFiles("$path/$item");
		}
	} else {
		$path =~ m|/([^/]*)$|;
		if (defined($Table{$1})) {
			Compare($Table{$1}, $path);
			delete $Table{$1};
		} elsif (defined($Sames{$1})) {
			push(@SameResult, $path);
		} else {
			push(@NotFound, $path);
		}
	}
}

if ($DirMode) { ParseFiles($Obj2); }
else { Compare($Obj1, $Obj2); }

print join("", @Diffs);
print join("", @Sames);

if (@SameResult) {
	print "%%%% multiple files of same name %%%%\n";
	for my $path (@SameResult) {
		print "(M2) $path\n";
		my ($name) = $path =~ m|/([^/]*)$|;
		for my $obj (@{$Sames{$name}}) {
			print "(M1) $obj\n";
		}
	}
}

if (%Table) {
	print "%%%% files (1) not found in (2) %%%%\n";
	for my $key (keys %Table) {
		print "(F1) $Table{$key}\n";
	}
}

if (@NotFound) {
	print "%%%% files (2) not found in (1) %%%%\n";
	for my $path (@NotFound) {
		print "(F2) $path\n";
	}
}
