#! /usr/bin/perl -w
use strict;
use utf8;

# 最後の引数 (フォルダ) にリンクを張る．
# 1) "指令:ファイル処理" メニューで "ファイル処理" ダイアログを開き，
# 2) このウィンドウ (MakeLink.pl) にリンクを張りたいファイルをマウスで落とし，
# 3) 最後にリンクを置きたいフォルダを落とし，
# 4) "ファイル処理" ダイアログの "実行" ボタンを押す．

# 相対経路を使う ($Relative = '' とすれば絶対径路を使う)
my $Relative = 1;
# シンボリックリンクを張る ($Symbolic = '' とすればハードリンクを張る)
my $Symbolic = 1;

die unless @ARGV;
my $TargetDir = pop(@ARGV);
$TargetDir =~ m|^/| or die "ERROR: pass me absolute path ";
my @stat = stat($TargetDir);
my $TargetVol = $stat[0];
while (my $file = shift) {
	$file =~ m|^/| or die "ERROR: pass me absolute path ";
	my ($name) = $file =~ m|([^/]*)/?$|;
	if (-e "$TargetDir/$name") {
		if (-e "$name.link") {
			warn("'$name' and '$name.link' already exist.\n");
			next;
		}
		$name .= '.link';
	}
	@stat = stat($file);
	my $samevol = $stat[0] == $TargetVol;
	my $symbol = ($Symbolic or not $samevol) ? '-s' : '';
	if ($Relative and $samevol) { # 相対経路
		my @src = split('/', $file);
		my @trg = split('/', $TargetDir);
		while (@src and @trg and $src[0] eq $trg[0]) {
			shift @src; shift @trg;
		}
		$file = join('/', @src);
		$file = ('../' x ($#trg + 1)) . $file if @trg;
		$file =~ s|/$||; # @src が空のとき
		$file = '.' unless (length($file)); # 自分自身へのリンク
	}
	system("ln $symbol '$file' '$TargetDir/$name'");
	print "$name -> $file\n";
}
