#!/usr/bin/perl -w
# DiagnoseFontsFolders.pl
use strict;
use utf8;
#use encoding "utf-8";
use Encode;

my @FontsDirs = ('/System/Library/Fonts', '/Library/Fonts', "$ENV{'HOME'}/Library/Fonts");
my %AllFiles;
my %Errors;
my @Table;

my $Japanese = $ENV{'UserLanguage'} eq 'Japanese';
sub Message($$) { return $Japanese ? $_[1] : $_[0]; }

my @ErrorLevel = (
	Message("Error", "想定外の誤り"),
	Message("Warning", "警告"),
	Message("Caution", "注意"),
	'???');

my $Err_CantOpendir		= Message(
	"0-Can't scan folder",
	"0-フォルダを走査できません");
my $Err_FontNotFound	= Message(
	"1-Standard TeX file not found",
	"1-TeX 標準のファイルがありません");
my $Err_BadFile			= Message(
	"1-File you should remove (Such a file disturbs to show fonts)",
	"1-除くべきファイル (これがあると正しく表示できません)");
my $Err_Subdir			= Message(
	"2-Fonts in a folder in Fonts folder may be ignored",
	"2-Fonts フォルダの中のフォルダの中身は使われないかもしれない");
my $Err_SameFonts		= Message(
	"2-Same fonts (May be normal)",
	"2-同一のフォント (通常は問題ありません)");
my $Err_DifferentNames	= Message(
	"2-Same fonts with different names (If tex related, you should remove it)",
	"2-別名の，同一と思われるフォント (TeX に関係する場合はどちらかを削除すべきです)");

my %BadFiles = ();
for (qw/AMS CM_1 CM_2 CM_3 CM_4 Euler LaTeX WNCY/)
{ $BadFiles{"$_.dfont"} = 1; }
my %PSFonts = ();
for (qw/CMBSY5 CMBSY7 CMMIB5 CMMIB7 EUEX10 EUFB5 EUFB7 EUFB10 EUFM5 EUFM7 EUFM10 EURB5 EURB7 EURB10 EURM5 EURM7 EURM10 EUSB5 EUSB7 EUSB10 EUSM5 EUSM7 EUSM10 MSAM5 MSAM7 MSAM10 MSBM5 MSBM7 MSBM10 WNCYB10 WNCYI10 WNCYR10 WNCYSC10 WNCYSS10 CMB10 CMBSY10 CMBX5 CMBX6 CMBX7 CMBX8 CMBX9 CMBX10 CMBX12 CMBXSL10 CMBXTI10 CMCSC10 CMDUNH10 CMEX10 CMFF10 CMFI10 CMFIB8 CMITT10 CMMI5 CMMI6 CMMI7 CMMI8 CMMI9 CMMI10 CMMI12 CMMIB10 CMR5 CMR6 CMR7 CMR8 CMR9 CMR10 CMR12 CMR17 CMSL8 CMSL9 CMSL10 CMSL12 CMSLTT10 CMSS8 CMSS9 CMSS10 CMSS12 CMSS17 CMSSBX10 CMSSDC10 CMSSI8 CMSSI9 CMSSI10 CMSSI12 CMSSI17 CMSSQ8 CMSSQI8 CMSY5 CMSY6 CMSY7 CMSY8 CMSY9 CMSY10 CMTCSC10 CMTEX8 CMTEX9 CMTEX10 CMTI7 CMTI8 CMTI9 CMTI10 CMTI12 CMTT8 CMTT9 CMTT10 CMTT12 CMU10 CMVTT10 LASY5 LASY6 LASY7 LASY8 LASY9 LASY10 LASYB10 LCIRCLE10 LCIRCLEW10 LCMSS8 LCMSSB8 LCMSSI8 LINE10 LINEW10 LOGO8 LOGO9 LOGO10 LOGOBF10 LOGOSL10/)
{ $PSFonts{$_} = -1; }
my %ScreenFonts = ('AMS:PS' => 'AMS/PS', 'CM:PS' => 'CM/PS', 'LATEX' => 'LaTeX');

sub Error($$) {
	my ($key, $val) = @_;
	push(@{$Errors{$key}}, $val);
}

sub ScanDir($);
sub ScanDir($) {
	my ($dir) = @_;
	opendir(DIR, $dir) or Error($Err_CantOpendir, $dir);
	my @items = readdir(DIR);
	closedir(DIR);
	for (@items) { $_ = decode('utf8', encode('iso-latin-1', $_)); }
	for (@items) {
		next if /^\./;
		if (-d "$dir/$_") {
			Error($Err_Subdir, "$dir/$_");
			ScanDir("$dir/$_");
		} else {
			$AllFiles{$_} .= "$dir\n";
		}
	}
}

print Message("Scan folders", "走査したフォルダ"), ":\n";
for (@FontsDirs) {
	print "\t$_\n";
	ScanDir($_);
}

my %AllNames;

for (sort keys %AllFiles) {
	my @dirs = split("\n", $AllFiles{$_});
	my $dirs = join(" + ", @dirs);
	Error($Err_SameFonts, "$_ in $dirs") if (@dirs > 1);
	push(@Table, "$_: $dirs");
	Error($Err_BadFile, "$_ in $dirs") if ($BadFiles{$_});
	my $name = $_;
	$PSFonts{$name} = '' if ($PSFonts{$name});
	$name =~ tr/a-z/A-Z/;
	$ScreenFonts{$1} = '' if ($name =~ /^(AMS:PS|CM:PS|LaTeX).* screen fonts/i);
	$name =~ s/\.\D*$//;
	$AllNames{$name} .= "$_\n";
}

for (sort keys %AllNames) {
	my @fonts = split("\n", $AllNames{$_});
	Error($Err_DifferentNames, join(" + ", @fonts)) if (@fonts > 1);
}

for (sort keys %ScreenFonts) {
	Error($Err_FontNotFound, "$ScreenFonts{$_} screen fonts") if ($ScreenFonts{$_});
}
for (sort keys %PSFonts) {
	Error($Err_FontNotFound, $_) if ($PSFonts{$_});
}

print "\n";
for (sort keys %Errors) {
	my ($level, $title) = $_ =~ /^(\d)-(.*)$/;
	$level = $ErrorLevel[$level];
	my @list = sort @{$Errors{$_}};
	print "$level: $title\n", join("\n", @list), "\n\n";
}
print "\n\n", Message("All fonts", "フォント一覧"), ":\n", join("\n", @Table);
print "\n-- end --\n";
